@@ -32,26 +32,36 @@
  */
 
 #include "_pylibmcmodule.h"
+#ifdef USE_ZLIB
+#  include <zlib.h>
+#  define ZLIB_BUFSZ (1 << 14)
+#  define _ZLIB_ERR(s, rc) \
+  PyErr_Format(PylibMCExc_MemcachedError, "zlib error %d in " s, rc);
+#endif
 
 
-/* {{{ _pylibmc.client implementation */
 /* {{{ Type methods */
 static PylibMC_Client *PylibMC_ClientType_new(PyTypeObject *type,
         PyObject *args, PyObject *kwds) {
     PylibMC_Client *self;
 
-    self = (PylibMC_Client *)type->tp_alloc(type, 0);
+    /* GenericNew calls GenericAlloc (via the indirection type->tp_alloc) which
+     * adds GC tracking if flagged for, and also calls PyObject_Init. */
+    self = (PylibMC_Client *)PyType_GenericNew(type, args, kwds);
+
     if (self != NULL) {
         self->mc = memcached_create(NULL);
-    } else {
-        args = kwds = NULL;
     }
 
     return self;
 }
 
 static void PylibMC_ClientType_dealloc(PylibMC_Client *self) {
-    memcached_free(self->mc);
+    if (self->mc != NULL) {
+        memcached_free(self->mc);
+    }
+
+    self->ob_type->tp_free(self);
 }
 /* }}} */
 
@@ -150,12 +160,156 @@ error:
     return -1;
 }
 
+/* {{{ Compression helpers */
+#ifdef USE_ZLIB
+static PyObject *_PylibMC_Deflate(PyObject *value) {
+    int rc;
+    char *out;
+    z_stream strm;
+    Py_ssize_t length, out_sz;
+
+    if (!PyString_Check(value)) {
+        return NULL;
+    }
+
+    /* Don't ask me about this one. Got it from zlibmodule.c in Python 2.6. */
+    length = PyString_GET_SIZE(value);
+    out_sz = length + length / 1000 + 12 + 1;
+
+    if ((out = PyMem_New(char, out_sz)) == NULL) {
+        return NULL;
+    }
+
+    strm.avail_in = length;
+    strm.avail_out = out_sz;
+    strm.next_in = (Byte *)PyString_AS_STRING(value);
+    strm.next_out = (Byte *)out;
+
+    strm.zalloc = (alloc_func)NULL;
+    strm.zfree = (free_func)Z_NULL;
+
+    /* TODO Expose compression level somehow. */
+    if ((rc = deflateInit((z_streamp)&strm, Z_BEST_SPEED)) != Z_OK) {
+        _ZLIB_ERR("deflateInit", rc);
+        goto error;
+    }
+
+    if ((rc = deflate((z_streamp)&strm, Z_FINISH)) != Z_STREAM_END) {
+        _ZLIB_ERR("deflate", rc);
+        goto error;
+    }
+
+    if ((rc = deflateEnd((z_streamp)&strm)) != Z_OK) {
+        _ZLIB_ERR("deflateEnd", rc);
+        goto error;
+    }
+
+    return PyString_FromStringAndSize(out, strm.total_out);
+error:
+    PyMem_Del(out);
+    return NULL;
+}
+
+static PyObject *_PylibMC_Inflate(char *value, size_t size) {
+    int rc;
+    char *out;
+    PyObject *out_obj;
+    Py_ssize_t rvalsz;
+    z_stream strm;
+
+    /* Output buffer */
+    rvalsz = ZLIB_BUFSZ;
+    out_obj = PyString_FromStringAndSize(NULL, rvalsz);
+    if (out_obj == NULL) {
+        return NULL;
+    }
+    out = PyString_AS_STRING(out_obj);
+
+    /* Set up zlib stream. */
+    strm.avail_in = size;
+    strm.avail_out = (uInt)rvalsz;
+    strm.next_in = (Byte *)value;
+    strm.next_out = (Byte *)out;
+
+    strm.zalloc = (alloc_func)NULL;
+    strm.zfree = (free_func)Z_NULL;
+
+    /* TODO Add controlling of windowBits with inflateInit2? */
+    if ((rc = inflateInit((z_streamp)&strm)) != Z_OK) {
+        _ZLIB_ERR("inflateInit", rc);
+        goto error;
+    }
+
+    do {
+        rc = inflate((z_streamp)&strm, Z_FINISH);
+
+        switch (rc) {
+        case Z_STREAM_END:
+            break;
+        /* When a Z_BUF_ERROR occurs, we should be out of memory.
+         * This is also true for Z_OK, hence the fall-through. */
+        case Z_BUF_ERROR:
+            if (strm.avail_out) {
+                _ZLIB_ERR("inflate", rc);
+                inflateEnd(&strm);
+                goto error;
+            }
+        /* Fall-through */
+        case Z_OK:
+            if (_PyString_Resize(&out_obj, (Py_ssize_t)(rvalsz << 1)) < 0) {
+                inflateEnd(&strm);
+                goto error;
+            }
+            /* Wind forward */
+            out = PyString_AS_STRING(out_obj);
+            strm.next_out = (Byte *)(out + rvalsz);
+            strm.avail_out = rvalsz;
+            rvalsz = rvalsz << 1;
+            break;
+        default:
+            inflateEnd(&strm);
+            goto error;
+        }
+    } while (rc != Z_STREAM_END);
+
+    if ((rc = inflateEnd(&strm)) != Z_OK) {
+        _ZLIB_ERR("inflateEnd", rc);
+        goto error;
+    }
+
+    _PyString_Resize(&out_obj, strm.total_out);
+    return out_obj;
+error:
+    Py_DECREF(out_obj);
+    return NULL;
+}
+#endif
+/* }}} */
+
 static PyObject *_PylibMC_parse_memcached_value(char *value, size_t size,
         uint32_t flags) {
     PyObject *retval, *tmp;
 
+#if USE_ZLIB
+    PyObject *inflated = NULL;
+
+    /* Decompress value if necessary. */
+    if (flags & PYLIBMC_FLAG_ZLIB) {
+        inflated = _PylibMC_Inflate(value, size);
+        value = PyString_AS_STRING(inflated);
+        size = PyString_GET_SIZE(inflated);
+    }
+#else
+    if (flags & PYLIBMC_FLAG_ZLIB) {
+        PyErr_SetString(PylibMCExc_MemcachedError,
+            "value for key compressed, unable to inflate");
+        return NULL;
+    }
+#endif
+
     retval = NULL;
-    switch (flags) {
+
+    switch (flags & PYLIBMC_FLAG_TYPES) {
         case PYLIBMC_FLAG_PICKLE:
             retval = _PylibMC_Unpickle(value, size);
             break;
@@ -178,6 +332,12 @@ static PyObject *_PylibMC_parse_memcached_value(char *value, size_t size,
                     "unknown memcached key flags %u", flags);
     }
 
+#if USE_ZLIB
+    if (inflated != NULL) {
+        Py_DECREF(inflated);
+    }
+#endif
+
     return retval;
 }
 
@@ -224,18 +384,26 @@ static PyObject *_PylibMC_RunSetCommand(PylibMC_Client *self,
     PyObject *retval = NULL;
     PyObject *store_val = NULL;
     unsigned int time = 0;
+    unsigned int min_compress = 0;
     uint32_t store_flags = 0;
 
-    static char *kws[] = { "key", "val", "time", NULL };
+    static char *kws[] = { "key", "val", "time", "min_compress_len", NULL };
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#O|I", kws,
-                &key, &key_sz, &val, &time)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#O|II", kws,
+                &key, &key_sz, &val, &time, &min_compress)) {
         return NULL;
     }
     if (!_PylibMC_CheckKeyStringAndSize(key, key_sz)) {
         return NULL;
     }
 
+#ifndef USE_ZLIB
+    if (min_compress) {
+        PyErr_SetString(PyExc_TypeError, "min_compress_len without zlib");
+        return NULL;
+    }
+#endif
+
     /* Adapt val to a str. */
     if (PyString_Check(val)) {
         store_val = val;
@@ -265,6 +433,22 @@ static PyObject *_PylibMC_RunSetCommand(PylibMC_Client *self,
         return NULL;
     }
 
+#ifdef USE_ZLIB
+    if (min_compress && PyString_GET_SIZE(store_val) > min_compress) {
+        PyObject *deflated = _PylibMC_Deflate(val);
+
+        if (deflated != NULL) {
+            Py_DECREF(store_val);
+            store_val = deflated;
+            store_flags |= PYLIBMC_FLAG_ZLIB;
+        } else {
+            /* XXX What to do here, really? */
+            PyErr_Print();
+            PyErr_Clear();
+        }
+    }
+#endif
+
     rc = f(self->mc, key, key_sz,
            PyString_AS_STRING(store_val), PyString_GET_SIZE(store_val),
            time, store_flags);
@@ -460,10 +644,13 @@ static PyObject *PylibMC_Client_get_multi(PylibMC_Client *self, PyObject *args,
     }
     Py_XDECREF(key_it);
 
-    if (i == 0) {
-        /* No usable keys to fetch. */
+    if (nkeys != 0 && i != nkeys) {
+        /* There were keys given, but some keys didn't pass validation. */
         nkeys = 0;
         goto cleanup;
+    } else if (nkeys == 0) {
+        retval = PyDict_New();
+        goto earlybird;
     } else if (PyErr_Occurred()) {
         nkeys--;
         goto cleanup;
@@ -517,6 +704,7 @@ static PyObject *PylibMC_Client_get_multi(PylibMC_Client *self, PyObject *args,
         free(curr_val);
     }
 
+earlybird:
     PyMem_Free(key_lens);
     PyMem_Free(keys);
     for (i = 0; i < nkeys; i++) {
@@ -527,6 +715,7 @@ static PyObject *PylibMC_Client_get_multi(PylibMC_Client *self, PyObject *args,
     /* Not INCREFing because the only two outcomes are NULL and a new dict.
      * We're the owner of that dict already, so. */
     return retval;
+
 cleanup:
     Py_XDECREF(retval);
     PyMem_Free(key_lens);
@@ -580,6 +769,7 @@ static PyObject *_PylibMC_DoMulti(PyObject *values, PyObject *func,
                 goto iter_error;
 
             args = PyTuple_Pack(2, key, value);
+            Py_DECREF(value);
         } else {
             args = PyTuple_Pack(1, key);
         }
@@ -676,6 +866,23 @@ static PyObject *PylibMC_Client_delete_multi(PylibMC_Client *self,
                 &keys, &PyInt_Type, &time, &prefix))
         return NULL;
 
+    /**
+     * Because of how DoMulti works, we have to prohibit the use of mappings
+     * here. Otherwise, the values of the mapping will be the second argument
+     * to the delete function; the time argument will then become a third
+     * argument, which delete doesn't take.
+     *
+     * So a mapping to DoMulti would produce calls like:
+     *   DoMulti({"a": 1, "b": 2}, time=3)
+     *      delete("a", 1, 3)
+     *      delete("b", 2, 3)
+     */
+    if (PyMapping_Check(keys)) {
+        PyErr_SetString(PyExc_TypeError,
+            "keys must be a sequence, not a mapping");
+        return NULL;
+    }
+
     if ((delete = PyObject_GetAttrString((PyObject *)self, "delete")) == NULL)
         return NULL;
 
@@ -689,6 +896,18 @@ static PyObject *PylibMC_Client_delete_multi(PylibMC_Client *self,
     }
     Py_DECREF(delete);
 
+    if (retval == NULL)
+        return NULL;
+
+    if (PyList_Size(retval) == 0) {
+        Py_DECREF(retval);
+        retval = Py_True;
+    } else {
+        Py_DECREF(retval);
+        retval = Py_False;
+    }
+    Py_INCREF(retval);
+
     return retval;
 error:
     Py_XDECREF(delete);
@@ -860,7 +1079,7 @@ static PyObject *PylibMC_Client_flush_all(PylibMC_Client *self,
 
     rc = memcached_flush(self->mc, expire);
     if (rc != MEMCACHED_SUCCESS)
-        return PylibMC_ErrFromMemcached(self, "delete_multi", rc);
+        return PylibMC_ErrFromMemcached(self, "flush_all", rc);
 
     Py_RETURN_TRUE;
 }
@@ -871,18 +1090,17 @@ static PyObject *PylibMC_Client_disconnect_all(PylibMC_Client *self) {
 }
 
 static PyObject *PylibMC_Client_clone(PylibMC_Client *self) {
-    PylibMC_Client *new_self;
+    /* Essentially this is a reimplementation of the allocator, only it uses a
+     * cloned memcached_st for mc. */
+    PylibMC_Client *clone;
 
-    /* XXX Is it really wise to short-circuit like this? I don't see a problem
-     * with it. */
-    new_self = (PylibMC_Client *)self->ob_type->tp_new(self->ob_type, NULL, NULL);
-    if (new_self == NULL) {
+    clone = (PylibMC_Client *)PyType_GenericNew(self->ob_type, NULL, NULL);
+    if (clone == NULL) {
         return NULL;
     }
-    memset(new_self->mc, 0, sizeof(memcached_st));
-    new_self->mc = memcached_clone(new_self->mc, self->mc);
 
-    return (PyObject *)new_self;
+    clone->mc = memcached_clone(NULL, self->mc);
+    return (PyObject *)clone;
 }
 /* }}} */
 
@@ -993,8 +1211,19 @@ PyMODINIT_FUNC init_pylibmc(void) {
     PyObject *module, *exc_objs;
     PylibMC_Behavior *b;
     PylibMC_McErr *err;
+    int libmemcached_version_minor;
     char name[128];
 
+    /* Check minimum requirement of libmemcached version */
+    libmemcached_version_minor = \
+        atoi(strchr(LIBMEMCACHED_VERSION_STRING, '.') + 1);
+    if (libmemcached_version_minor < 32) {
+        PyErr_Format(PyExc_RuntimeError,
+            "pylibmc requires >= libmemcached 0.32, was compiled with %s",
+            LIBMEMCACHED_VERSION_STRING);
+        return;
+    }
+
     if (PyType_Ready(&PylibMC_ClientType) < 0) {
         return;
     }
@@ -1025,6 +1254,14 @@ Oh, and: plankton.\n");
 
     PyModule_AddStringConstant(module, "__version__", PYLIBMC_VERSION);
 
+#ifdef USE_ZLIB
+    Py_INCREF(Py_True);
+    PyModule_AddObject(module, "support_compression", Py_True);
+#else
+    Py_INCREF(Py_False);
+    PyModule_AddObject(module, "support_compression", Py_False);
+#endif
+
     PylibMCExc_MemcachedError = PyErr_NewException(
             "_pylibmc.MemcachedError", NULL, NULL);
     PyModule_AddObject(module, "MemcachedError",
