@@ -1,31 +1,64 @@
-"""`python-memcached`-compatible wrapper around `_pylibmc`.
+"""Snappy libmemcached wrapper
 
-The interface is pretty much exactly the same as python-memcached, with some
-minor differences. If you should happen to spot any, file a bug!
+pylibmc is a Python wrapper around TangentOrg's libmemcached library.
 
->>> import pylibmc
->>> mc = pylibmc.Client(["127.0.0.1"])
->>> b = mc.behaviors
->>> ks = list(sorted(k for k in b.keys() if not k.startswith("_")))
->>> ks  # doctest: +NORMALIZE_WHITESPACE
-['buffer_requests', 'cache_lookups', 'connect_timeout', 'distribution', 'hash',
- 'ketama', 'ketama_hash', 'ketama_weighted', 'no block', 'poll_timeout',
- 'rcv_timeout', 'retry_timeout', 'server_failure_limit', 'snd_timeout',
- 'socket recv size', 'socket send size', 'sort_hosts', 'support_cas',
- 'tcp_nodelay', 'verify_key']
->>> b["hash"]
-'default'
->>> b["hash"] = 'fnv1a_32'
->>> mc.behaviors["hash"]
-'fnv1a_32'
->>> super(pylibmc.Client, mc).get_behaviors()["hash"]
-6
+The interface is intentionally made as close to python-memcached as possible,
+so that applications can drop-in replace it.
+
+Example usage
+=============
+
+Create a connection and configure it::
+
+    >>> import pylibmc
+    >>> mc = pylibmc.Client(["127.0.0.1"], binary=True)
+    >>> mc.behaviors = {"tcp_nodelay": True, "ketama": True}
+
+Basic operation::
+
+    >>> mc.set("some_key", "Some value")
+    True
+    >>> value = mc.get("some_key")
+    >>> value
+    'Some value'
+    >>> mc.set("another_key", 3)
+    True
+    >>> mc.delete("another_key")
+    True
+    >>> mc.set("key", "1")  # str or int is fine
+    True
+
+Atomic increments and decrements::
+
+    >>> mc.incr("key")
+    2L
+    >>> mc.decr("key")
+    1L
+
+Batch operation::
+
+    >>> mc.get_multi(["key", "another_key"])
+    {'key': '1'}
+    >>> mc.set_multi({"cats": ["on acid", "furry"], "dogs": True})
+    []
+    >>> mc.get_multi(["cats", "dogs"])
+    {'cats': ['on acid', 'furry'], 'dogs': True}
+    >>> mc.delete_multi(["cats", "dogs", "nonextant"])
+    False
+
+Further Reading
+===============
+
+See http://sendapatch.se/projects/pylibmc/
 """
 
 import _pylibmc
+from warnings import warn
+from Queue import Queue
 
 __all__ = ["hashers", "distributions", "Client"]
 __version__ = _pylibmc.__version__
+support_compression = _pylibmc.support_compression
 
 errors = tuple(e for (n, e) in _pylibmc.exceptions)
 # *Cough* Uhm, not the prettiest of things but this unpacks all exception
@@ -99,10 +132,10 @@ class Client(_pylibmc.client):
         Reverses the integer constants for `hash` and `distribution` into more
         understandable string values. See *set_behaviors* for info.
         """
-        behaviors = super(Client, self).get_behaviors()
-        behaviors["hash"] = hashers_rvs[behaviors["hash"]]
-        behaviors["distribution"] = distributions_rvs[behaviors["distribution"]]
-        return BehaviorDict(self, behaviors)
+        bvrs = super(Client, self).get_behaviors()
+        bvrs["hash"] = hashers_rvs[bvrs["hash"]]
+        bvrs["distribution"] = distributions_rvs[bvrs["distribution"]]
+        return BehaviorDict(self, bvrs)
 
     def set_behaviors(self, behaviors):
         """Sets the behaviors on the underlying C client instance.
@@ -115,17 +148,27 @@ class Client(_pylibmc.client):
         This also happens for `distribution`.
         """
         behaviors = behaviors.copy()
+        if any(" " in k for k in behaviors):
+            warn(DeprecationWarning("space-delimited behavior names "
+                                    "are deprecated"))
+            for k in [k for k in behaviors if " " in k]:
+                behaviors[k.replace(" ", "_")] = behaviors.pop(k)
         if behaviors.get("hash") is not None:
             behaviors["hash"] = hashers[behaviors["hash"]]
+        if behaviors.get("ketama_hash") is not None:
+            behaviors["ketama_hash"] = hashers[behaviors["ketama_hash"]]
         if behaviors.get("distribution") is not None:
             behaviors["distribution"] = distributions[behaviors["distribution"]]
         return super(Client, self).set_behaviors(behaviors)
 
     behaviors = property(get_behaviors, set_behaviors)
+    @property
+    def behaviours(self):
+        raise AttributeError("nobody uses british spellings")
 
 from contextlib import contextmanager
 
-class ClientPool(list):
+class ClientPool(Queue):
     """Client pooling helper.
 
     This is mostly useful in threaded environments, because a client isn't
@@ -145,19 +188,28 @@ class ClientPool(list):
     True
     """
 
+    def __init__(self, mc=None, n_slots=None):
+        Queue.__init__(self, n_slots)
+        if mc is not None:
+            self.fill(mc, n_slots)
+
     @contextmanager
-    def reserve(self):
-        """Reserve a client, and put it back into the pool when done."""
-        mc = self.pop()
+    def reserve(self, timeout=None):
+        """Context manager for reserving a client from the pool.
+
+        If *timeout* is given, it specifiecs how long to wait for a client to
+        become available.
+        """
+        mc = self.get(True, timeout=timeout)
         try:
             yield mc
         finally:
-            self.append(mc)
+            self.put(mc)
 
     def fill(self, mc, n_slots):
         """Fill *n_slots* of the pool with clones of *mc*."""
         for i in xrange(n_slots):
-            self.append(mc.clone())
+            self.put(mc.clone())
 
 class ThreadMappedPool(dict):
     """Much like the *ClientPool*, helps you with pooling.
