@@ -36,7 +36,7 @@ Multi functionality.
 >>> c.get_multi("abc").keys() == ["a", "c", "b"]
 True
 >>> c.delete_multi("abc")
-[]
+True
 >>> c.get_multi("abc").keys() == []
 True
 >>> c.set_multi(dict(zip("abc", "def")), key_prefix="test_")
@@ -46,7 +46,7 @@ True
 >>> c.get("test_a")
 'd'
 >>> c.delete_multi("abc", key_prefix="test_")
-[]
+True
 >>> bool(c.get_multi("abc", key_prefix="test_"))
 False
 
@@ -54,9 +54,13 @@ Zero-key-test-time!
 >>> c.get_multi([""])
 {}
 >>> c.delete_multi([""])
-['']
+False
 >>> c.set_multi({"": "hi"})
 ['']
+>>> c.delete_multi({"a": "b"})
+Traceback (most recent call last):
+  ...
+TypeError: keys must be a sequence, not a mapping
 
 Timed stuff. The reason we, at UNIX times, set it two seconds in the future and
 then sleep for >3 is that memcached might round the time up and down and left
@@ -94,6 +98,10 @@ Traceback (most recent call last):
   ...
 TypeError: key must be an instance of str
 
+This didn't use to work, but now it does.
+>>> c.get_multi([])
+{}
+
 Getting stats is fun!
 >>> for (svr, stats) in c.get_stats():
 ...     print svr
@@ -187,23 +195,35 @@ Python-wrapped behaviors dict
 >>> pc.behaviors.update({"hash": "fnv1a_32", "distribution": "consistent"})
 >>> (pc.behaviors["hash"], pc.behaviors["distribution"])
 ('fnv1a_32', 'consistent')
+
+>>> pc = pylibmc.Client(["%s:%d" % test_server[1:]])
+>>> b = pc.behaviors
+>>> ks = list(sorted(k for k in b.keys() if not k.startswith("_")))
+>>> ks  # doctest: +NORMALIZE_WHITESPACE
+['buffer_requests', 'cache_lookups', 'cas', 'connect_timeout', 'distribution',
+ 'failure_limit', 'hash', 'ketama', 'ketama_hash', 'ketama_weighted',
+ 'no_block', 'receive_timeout', 'send_timeout', 'tcp_nodelay', 'verify_keys']
+>>> b["hash"]
+'default'
+>>> b["hash"] = 'fnv1a_32'
+>>> pc.behaviors["hash"]
+'fnv1a_32'
+>>> super(pylibmc.Client, pc).get_behaviors()["hash"]
+6
 """
 
 # Used to test pickling.
 class Foo(object): pass
 
-# Fix up sys.path so as to include the build/lib.*/ directory.
+# Fix up sys.path so as to include the correct build/lib.*/ directory.
 import sys
-import os
-from glob import glob
+from distutils.dist import Distribution
+from distutils.command.build import build
 
-dist_dir = os.path.dirname(__file__)
-for build_dir in glob(os.path.join(dist_dir, "build", "lib.*")):
-    sys.path.insert(0, build_dir)
-    break
-else:
-    print >>sys.stderr, "Using system-wide installation of pylibmc!"
-    print >>sys.stderr, "==========================================\n"
+build_cmd = build(Distribution({"ext_modules": True}))
+build_cmd.finalize_options()
+lib_dirn = build_cmd.build_lib
+sys.path.insert(0, lib_dirn)
 
 import pylibmc, _pylibmc
 import socket
@@ -230,8 +250,14 @@ class TestCmemcached(unittest.TestCase):
         self.assertEqual(self.mc.get("str12345"), None)
         self.assertEqual(ret, True)
 
-        ret = self.mc.delete("hello world")
-        self.assertEqual(ret, False)
+        # This test only works with old memcacheds. This has become a "client
+        # error" in memcached.
+        try:
+            ret = self.mc.delete("hello world")
+        except pylibmc.ClientError:
+            pass
+        else:
+            self.assertEqual(ret, False)
 
     def testGetMulti(self):
         self.mc.set("a", "valueA")
@@ -339,9 +365,13 @@ class TestProgram(unittest.TestProgram):
         return runner.run(self.test)
 
 if __name__ == "__main__":
-    print "Starting tests with _pylibmc at", _pylibmc.__file__
+    if hasattr(_pylibmc, "__file__"):
+        print "Starting tests with _pylibmc at", _pylibmc.__file__
+    else:
+        print "Starting tests with static _pylibmc:", _pylibmc
     print "Reported libmemcached version:", _pylibmc.libmemcached_version
     print "Reported pylibmc version:", _pylibmc.__version__
+    print "Support compression:", _pylibmc.support_compression
 
     if not is_alive(test_server):
         raise SystemExit("Test server (%r) not alive." % (test_server,))
